package pps.core.prediction.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.prediction.entity.PlantPredictedPowerDataEnt;
import pps.core.prediction.mapper.PlantPredictedPowerDataMapper;
import pps.core.prediction.service.data.plant_predicted_power_data.GetPlantPredictedPowerDataInput;
import pps.core.prediction.service.data.plant_predicted_power_data.GetPlantPredictedPowerDataOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.feature.api.annotation.XApiGet;

import java.util.Date;
import java.util.List;


@XService
@XText("电站预测发电数据表")
public class PlantPredictedPowerDataService {

    @XApiGet
    @XText("根据电站预测4小时内的预测数据")
    public XListResult<GetPlantPredictedPowerDataOutput> getFourPlantPredictedPowerData(XContext context, GetPlantPredictedPowerDataInput input){
        PlantPredictedPowerDataMapper mapper = context.getBean(PlantPredictedPowerDataMapper.class);
        Date date = new Date();
        QueryWrapper<PlantPredictedPowerDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PlantPredictedPowerDataEnt::getPlantId ,input.getPlantId());
        if(XStringUtils.isEmpty(input.getStartTime())){
            queryWrapper.lambda().ge(PlantPredictedPowerDataEnt::getDataDate , date);
            queryWrapper.lambda().le(PlantPredictedPowerDataEnt::getDataDate , XDateUtils.addHours(date , 4));
        } else{
            String dataDate = XDateUtils.getDateString(new Date()) + " " + input.getStartTime() + ":00";
            queryWrapper.lambda().ge(PlantPredictedPowerDataEnt::getDataDate , dataDate);
            queryWrapper.lambda().le(PlantPredictedPowerDataEnt::getDataDate , XDateUtils.addHours(XDateUtils.parse(dataDate) , 4));
        }
        queryWrapper.lambda().orderByAsc(PlantPredictedPowerDataEnt::getDataDate);
        List<PlantPredictedPowerDataEnt> list = mapper.selectList(queryWrapper);
        List<GetPlantPredictedPowerDataOutput> outputs = XCopyUtils.copyNewList(list , GetPlantPredictedPowerDataOutput.class);
        outputs.forEach(item->{
            item.setPredictedPower(item.getPower());
        });
        return XListResult.success(outputs);
    }

    @XApiGet
    @XText("根据电站预测7天的预测数据")
    public XListResult<GetPlantPredictedPowerDataOutput> getSevenPlantPredictedPowerData(XContext context, GetPlantPredictedPowerDataInput input){
        PlantPredictedPowerDataMapper mapper = context.getBean(PlantPredictedPowerDataMapper.class);
        Date date = new Date();
        QueryWrapper<PlantPredictedPowerDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PlantPredictedPowerDataEnt::getPlantId ,input.getPlantId());
        if(XStringUtils.isEmpty(input.getStartTime())){
            queryWrapper.lambda().ge(PlantPredictedPowerDataEnt::getDataDate , date);
            queryWrapper.lambda().le(PlantPredictedPowerDataEnt::getDataDate , XDateUtils.addDays(date , 7));
        } else{
            queryWrapper.lambda().ge(PlantPredictedPowerDataEnt::getDataDate , input.getStartTime());
            queryWrapper.lambda().le(PlantPredictedPowerDataEnt::getDataDate , XDateUtils.addDays(XDateUtils.parse(input.getStartTime()) , 7));
        }
        queryWrapper.lambda().orderByAsc(PlantPredictedPowerDataEnt::getDataDate);
        List<PlantPredictedPowerDataEnt> list = mapper.selectList(queryWrapper);
        List<GetPlantPredictedPowerDataOutput> outputs = XCopyUtils.copyNewList(list , GetPlantPredictedPowerDataOutput.class);
        outputs.forEach(item->{
            item.setPredictedPower(item.getPower());
        });
        return XListResult.success(outputs);
    }
}
